<?xml version="1.0" encoding="utf-8"?>

<!-- Copyright (c) 2007 Chris Povirk -->
<!-- Distributed under the terms of the GNU General Public License v2 -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:tag="urn:X-tags:twofoos.org" xmlns:xi="http://www.w3.org/2001/XInclude" xmlns="http://www.w3.org/1999/xhtml" exclude-result-prefixes="tag #default xi">
<xsl:output method="xml" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" encoding="utf-8" indent="yes" />

  <xsl:template match="/page">

    <html lang="en" xml:lang="en">
    <head>
      <title>
        <xsl:value-of select="//title" />
        <xsl:if test="string(//subtitle)">
          <xsl:text>: </xsl:text>
          <xsl:value-of select="//subtitle" />
        </xsl:if>
        <xsl:text> | Two 〈Foo〉s Walk into a 〈Bar〉</xsl:text>
      </title>

      <xi:include href="generatormeta.xml" />

      <link rel="stylesheet" type="text/css" href="/stylesheet.css" />

      <link rel="icon" type="image/png" href="/favicon.ico" />

      <script type="text/javascript" src="/highlightlatest.js" />

      <link rel="alternate" type="application/atom+xml" title="Two 〈Foo〉s Walk into a 〈Bar〉" href="/feed/" />

      <xsl:apply-templates select="//tags" mode="feedlinks" />

      <link rel="alternate" type="application/atom+xml" href="feed/">
        <xsl:attribute name="title">
          <xsl:text>Two 〈Foo〉s Walk into a 〈Bar〉 » </xsl:text>
          <xsl:value-of select="//title" />
        </xsl:attribute>
      </link>

      <xsl:apply-templates select="//head" />
    </head>
    <body>

      <div id="updateanchors"><xsl:apply-templates select="//updates" mode="anchors" /></div>

      <xi:include href="h1.xml" />

      <div id="content">
        <h2>
          <xsl:value-of select="//title" />
          <xsl:if test="string(//subtitle)">
            <xsl:text> </xsl:text>
            <small>
              <xsl:value-of select="//subtitle" />
            </small>
          </xsl:if>
        </h2>

        <xsl:apply-templates select="//content" mode="content" />
      </div>

      <div id="sidebar">
        <h2>Page Info</h2>

        <div id="tags">Tags: <xsl:apply-templates select="//tags" mode="taglist" /></div> 

        <div id="update">Last update: <xsl:apply-templates select="//updates" mode="latest" /></div> 

        <ul id="contents">
          <xsl:apply-templates select="//content" mode="tableofcontents" />
        </ul>

        <h2>Site Navigation/Info</h2>

        <xi:include href="nav.xml" />

        <xi:include href="footer.xml" />
      </div>

    </body>
    </html>

  </xsl:template>

  <xsl:template match="//head">
    <xsl:for-each select="*">
      <xsl:copy-of select="." />
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="//edit" mode="content">
    <ins datetime="{@datetime}"><xsl:apply-templates mode="content" /></ins>
  </xsl:template>

  <xsl:template match="//section" mode="content">
    <div><xsl:apply-templates mode="content" /></div>
  </xsl:template>

  <xsl:template match="//h" mode="anchor">
    <a id="{@anchor}" />
  </xsl:template>

  <xsl:template match="//h" mode="content">
    <xsl:call-template name="level7" />
  </xsl:template>

  <xsl:template name="level7">
    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section/ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <xsl:message terminate="yes">Section nesting too deep</xsl:message>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level6" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level6">
    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <h6>
          <xsl:apply-templates select="." mode="anchor" />
          <xsl:apply-templates mode="content" />
        </h6>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level5" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level5">
    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <h5>
          <xsl:apply-templates select="." mode="anchor" />
          <xsl:apply-templates mode="content" />
        </h5>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level4" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level4">
    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <h4>
          <xsl:apply-templates select="." mode="anchor" />
          <xsl:apply-templates mode="content" />
        </h4>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level3" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level3">
    <h3>
      <xsl:apply-templates select="." mode="anchor" />
      <xsl:apply-templates mode="content" />
    </h3>
  </xsl:template>

  <xsl:template match="//updates" mode="latest">
    <xsl:for-each select="//update">
      <xsl:sort select="@datetime" />

      <xsl:if test="position() = last()">
        <xsl:value-of select="translate(substring(@datetime, 0, 11),'-','‐')" />
        <xsl:text>: </xsl:text>
        <xsl:value-of select="." />
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="//updates" mode="anchors">
    <xsl:for-each select="//update">
      <xsl:sort select="@datetime" />

      <a id="update_{@datetime}" />
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="//tags" mode="taglist">
    <ul>
      <xsl:for-each select="*">

        <xsl:sort select="local-name()" />

        <li>
          <a href="/tag/{local-name()}/">
            <xsl:value-of select="local-name()" />
          </a>
        </li>

        <xsl:if test="position() = last()">
          <xsl:value-of select="." />
        </xsl:if>

      </xsl:for-each>
    </ul>
  </xsl:template>

  <xsl:template match="//tags" mode="feedlinks">
    <xsl:for-each select="*">
      <xsl:sort select="local-name()" />

      <link rel="alternate" type="application/atom+xml" title="Two 〈Foo〉s Walk into a 〈Bar〉 » {local-name()}" href="/tag/{local-name()}/feed/" />
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="//title" />

  <!-- Start Table of Contents -->

  <xsl:template match="//content" mode="tableofcontents">
    <xsl:apply-templates select="*" mode="lookingforh" />
  </xsl:template>

  <xsl:template match="//section" mode="lookingforh" />

  <xsl:template match="//h" mode="lookingforh">
    <li>
      <a href="#{@anchor}">
        <xsl:value-of select="." />
      </a>

      <xsl:apply-templates select="following-sibling::section[1]" mode="lookingforsection" />
    </li>
  </xsl:template>

  <!-- No child sections. -->
  <xsl:template match="//section[not(h)]" mode="lookingforsection" />

  <!-- There are child sections. -->
  <xsl:template match="//section[(h)]" mode="lookingforsection">
    <ul>
      <xsl:apply-templates select="*" mode="lookingforh" />
    </ul>
  </xsl:template>

  <xsl:template match="text()" mode="lookingforsection" />
  <xsl:template match="text()" mode="lookingforh" />

  <xsl:template match="*" mode="lookingforh">
    <xsl:apply-templates mode="lookingforh" />
  </xsl:template>

  <xsl:template match="*" mode="lookingforsection">
    <xsl:apply-templates mode="lookingforsection" />
  </xsl:template>

  <!-- End Table of Contents -->

  <xsl:template match="//content" mode="content">
    <xsl:apply-templates mode="content" />
  </xsl:template>

  <xsl:template match="*" mode="content">
<!--    <xsl:copy-of select="." /> -->

    <!-- Still includes xmlns="" -->
<!--    <xsl:copy>
      <xsl:copy-of select="@*" />
      <xsl:apply-templates mode="content" />
    </xsl:copy> -->
<!--    <xsl:element name="{local-name()}"><xsl:apply-templates mode="content" /></xsl:element> --> <!-- Doesn't include attributes; could probably fiddle with that.  Also, still includes xmlns="" -->

    <xsl:element name="{local-name()}">
      <xsl:copy-of select="@*" />
      <xsl:apply-templates mode="content" />
    </xsl:element>

<!--    <xsl:copy-of select="." copy-namespaces="no" /> -->
  </xsl:template>

</xsl:stylesheet>
