<?xml version="1.0" encoding="utf-8"?>

<!-- Copyright (c) 2007 Chris Povirk -->
<!-- Distributed under the terms of the GNU General Public License v2 -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:tag="urn:X-tags:twofoos.org" xmlns:xhtml="http://www.w3.org/1999/xhtml" xmlns:xi="http://www.w3.org/2001/XInclude" xmlns="http://www.w3.org/2005/Atom" exclude-result-prefixes="tag xi #default">
<xsl:output method="xml" encoding="utf-8" indent="yes" />

  <xsl:template match="/pages">
    <feed xml:lang="en">

      <title>Two 〈Foo〉s Walk into a 〈Bar〉</title>
      <subtitle>Featuring the Highest Angle Bracket–to–Word Ratio Among Site Titles</subtitle>

      <updated>
        <xsl:for-each select="//update">
          <xsl:sort select="@datetime" />

          <xsl:if test="position() = last()">
            <xsl:value-of select="@datetime" />
          </xsl:if>
        </xsl:for-each>
      </updated>

      <author>
        <name>Chris Povirk</name>
        <email>beigetangerine@gmail.com</email>
        <uri><xsl:value-of select="$basehref" /></uri>
      </author>

      <link rel="alternate" type="application/xhtml+xml" hreflang="en" href="{$basehref}" />
      <link rel="self" type="application/atom+xml" href="{$basehref}feed/" />

      <rights>Copyright © 2003–2007 Chris Povirk.  All Rights Reserved.</rights>

      <id><xsl:value-of select="$basehref" /></id>

      <generator uri="{$basehref}content/splat/" version="1.0">SPLAT</generator>

      <xsl:for-each select="//page//update">
        <xsl:sort select="@datetime" />

        <xsl:apply-templates select="." />
      </xsl:for-each>

    </feed>
  </xsl:template>

  <xsl:template match="//page//update">
    <entry>
      <xsl:variable name="datetime"><xsl:value-of select="@datetime" /></xsl:variable>

      <updated><xsl:value-of select="@datetime" /></updated>

      <xsl:variable name="updatesummary">
        <xsl:value-of select="." />
      </xsl:variable>

      <summary>
        <xsl:choose>
          <xsl:when test="$updatesummary = 'initial posting'">
            New: <xsl:value-of select="ancestor::page/description" />
          </xsl:when>

          <xsl:otherwise>
            Update: <xsl:value-of select="$updatesummary" />
          </xsl:otherwise>
        </xsl:choose>
      </summary>

      <xsl:for-each select="ancestor::page">

        <xsl:for-each select="title">
          <title><xsl:value-of select="." /></title>
        </xsl:for-each>

        <xsl:variable name="href">
          <xsl:value-of select="$basehref" />
          <xsl:text>content/</xsl:text>
          <xsl:value-of select="ancestor::file/@name" />
          <xsl:text>/#update_</xsl:text>
          <xsl:value-of select="$datetime" />
        </xsl:variable>

        <link rel="alternate" type="application/xhtml+xml" href="{$href}" />

        <id><xsl:value-of select="$href" /></id>

        <xsl:for-each select="tags/*">
          <xsl:sort select="local-name()" />

          <category term="{local-name()}" />
        </xsl:for-each> 

        <content type="xhtml" xml:base="{$basehref}">
          <div xmlns="http://www.w3.org/1999/xhtml">
            <h2><xsl:value-of select="$updatesummary" /></h2>

            <xsl:apply-templates select="content/*" mode="lookingforupdate">
              <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
            </xsl:apply-templates>
          </div>
        </content>

      </xsl:for-each> 
    </entry>
  </xsl:template>

<!--  <xsl:template match="//edit">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:choose>
      <xsl:when test="@datetime=$datetime"> 
        <xhtml:ins datetime="{@datetime}" style="color: #000000"><xsl:apply-templates /></xhtml:ins>
      </xsl:when>

      <xsl:otherwise>
        <xsl:apply-templates />
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>-->

  <xsl:template match="//section" mode="insideupdate">
    <xsl:param name="datetime">null</xsl:param>

    <xhtml:div>
      <xsl:apply-templates mode="insideupdate">
        <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
      </xsl:apply-templates>
    </xhtml:div>
  </xsl:template>








  <xsl:template match="//h" mode="insideupdate">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:call-template name="level7">
      <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="level7">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section/ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <xsl:message terminate="yes">Section nesting too deep</xsl:message>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level6">
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level6">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <xhtml:h6>
          <xsl:apply-templates mode="insideupdate">
            <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
          </xsl:apply-templates>
        </xhtml:h6>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level5">
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level5">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section/ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <xhtml:h5>
          <xsl:apply-templates mode="insideupdate">
            <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
          </xsl:apply-templates>
        </xhtml:h5>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level4">
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level4">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:variable name="isNestedAtLevel" select="count(ancestor::section)" />

    <xsl:choose>
      <xsl:when test="$isNestedAtLevel &gt; 0">
        <xhtml:h4>
          <xsl:apply-templates mode="insideupdate">
            <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
          </xsl:apply-templates>
        </xhtml:h4>
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="level3">
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="level3">
    <xsl:param name="datetime">null</xsl:param>

    <xhtml:h3>
      <xsl:apply-templates mode="insideupdate">
        <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
      </xsl:apply-templates>
    </xhtml:h3>
  </xsl:template>





<!--  <xsl:template match="*">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:element name="{local-name()}" namespace="http://www.w3.org/1999/xhtml">
    <!- - If I'm going to do this, I need to search ancestors for ins - ->
<!- -      <xhtml:del> - ->
        <xsl:copy-of select="@*" />
        <xsl:apply-templates>
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:apply-templates>
<!- -      </xhtml:del> - ->
    </xsl:element>
  </xsl:template>-->

  <xsl:template match="text()" mode="lookingforupdate" />

  <xsl:template match="*" mode="lookingforupdate">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:choose>
      <xsl:when test="@datetime = $datetime">
        <xhtml:div>
          <xsl:apply-templates select="*|text()" mode="insideupdate">
            <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
          </xsl:apply-templates>
        </xhtml:div>
      </xsl:when>

      <xsl:otherwise>
        <xsl:apply-templates select="*" mode="lookingforupdate">
          <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="text()" mode="insideupdate">
    <xsl:value-of select="." />
  </xsl:template>

  <xsl:template match="edit" mode="insideupdate">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:apply-templates select="*|text()" mode="insideupdate">
      <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*" mode="insideupdate">
    <xsl:param name="datetime">null</xsl:param>

    <xsl:element name="{local-name()}" namespace="http://www.w3.org/1999/xhtml">
      <xsl:copy-of select="@*" />
      <xsl:apply-templates select="*|text()" mode="insideupdate">
        <xsl:with-param name="datetime"><xsl:value-of select="$datetime" /></xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>

</xsl:stylesheet>
